@@ -77,7 +77,7 @@ mutt_copy_hdr (FILE *in, FILE *out, LOFF_T off_start, LOFF_T off_end, int flags,
       if ((fgets (buf, sizeof (buf), in)) == NULL)
 	break;
 
-      /* Is it the begining of a header? */
+      /* Is it the beginning of a header? */
       if (nl && buf[0] != ' ' && buf[0] != '\t')
       {
 	ignore = 1;
@@ -147,7 +147,7 @@ mutt_copy_hdr (FILE *in, FILE *out, LOFF_T off_start, LOFF_T off_end, int flags,
     if ((fgets (buf, sizeof (buf), in)) == NULL)
       break;
 
-    /* Is it the begining of a header? */
+    /* Is it the beginning of a header? */
     if (nl && buf[0] != ' ' && buf[0] != '\t')
     {
       /* Do we have anything pending? */
@@ -868,6 +868,7 @@ static int address_header_decode (char **h)
   int l, rp = 0;
 
   ADDRESS *a = NULL;
+  ADDRESS *cur = NULL;
 
   switch (tolower ((unsigned char) *s))
   {
@@ -937,6 +938,9 @@ static int address_header_decode (char **h)
   
   mutt_addrlist_to_local (a);
   rfc2047_decode_adrlist (a);
+  for (cur = a; cur; cur = cur->next)
+    if (cur->personal)
+      rfc822_dequote_comment (cur->personal);
 
   /* angle brackets for return path are mandated by RfC5322,
    * so leave Return-Path as-is */
