@@ -53,7 +53,7 @@
 #include "lib.h"
 
 
-static struct sysexits
+static const struct sysexits
 {
   int v;
   const char *str;
@@ -548,7 +548,6 @@ static int mutt_mkwrapdir (const char *path, char *newfile, size_t nflen,
   const char *basename;
   char parent[_POSIX_PATH_MAX];
   char *p;
-  int rv;
 
   strfcpy (parent, NONULL (path), sizeof (parent));
   
@@ -563,17 +562,19 @@ static int mutt_mkwrapdir (const char *path, char *newfile, size_t nflen,
     basename = path;
   }
 
-  do 
+  snprintf (newdir, ndlen, "%s/%s", parent, ".muttXXXXXX");
+  if (mkdtemp(newdir) == NULL)
   {
-    snprintf (newdir, ndlen, "%s/%s", parent, ".muttXXXXXX");
-    mktemp (newdir);
-  } 
-  while ((rv = mkdir (newdir, 0700)) == -1 && errno == EEXIST);
+      dprint(1, (debugfile, "mutt_mkwrapdir: mkdtemp() failed\n"));
+      return -1;
+  }
   
-  if (rv == -1)
-    return -1;
-  
-  snprintf (newfile, nflen, "%s/%s", newdir, NONULL(basename));
+  if (snprintf (newfile, nflen, "%s/%s", newdir, NONULL(basename)) >= nflen)
+  {
+      rmdir(newdir);
+      dprint(1, (debugfile, "mutt_mkwrapdir: string was truncated\n"));
+      return -1;
+  }
   return 0;  
 }
 
@@ -696,7 +697,7 @@ FILE *safe_fopen (const char *path, const char *mode)
     return (fopen (path, mode));
 }
 
-static char safe_chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+@{}._-:%/";
+static const char safe_chars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+@{}._-:%/";
 
 void mutt_sanitize_filename (char *f, short slash)
 {
@@ -711,7 +712,7 @@ void mutt_sanitize_filename (char *f, short slash)
 
 /* these characters must be escaped in regular expressions */
 
-static char rx_special_chars[] = "^.[$()|*+?{\\";
+static const char rx_special_chars[] = "^.[$()|*+?{\\";
 
 int mutt_rx_sanitize_string (char *dest, size_t destlen, const char *src)
 {
@@ -786,7 +787,7 @@ char *mutt_read_line (char *s, size_t *size, FILE *fp, int *line, int flags)
       }
       else
       {
-        ungetc (c, fp); /* undo our dammage */
+        ungetc (c, fp); /* undo our damage */
         /* There wasn't room for the line -- increase ``s'' */
         offset = *size - 1; /* overwrite the terminating 0 */
         *size += STRING;
